/*
 * Disparo.java
 *
 * Created on 20 de abril de 2006, 20:44
 */

package Flier_Match.FMCliente.motion;
import java.awt.Color;

import Flier_Match.FMCliente.sonido.ManejadorSonidos;

import com.threed.jpct.Camera;
import com.threed.jpct.Object3D;
import com.threed.jpct.SimpleVector;
import com.threed.jpct.World;
/**
 *
 * @author  Luis Rocha
 * @version 
 */
public class Disparo {
	
	private SimpleVector posicion;
	private SimpleVector direccion;
	
	private Bullet bala;
	
	private ManejadorSonidos SoundManager;
	
	public static final int SONIDO_INICIAL=1;
	public static final int SONIDO_IMPACTO=2;
	public static final int SONIDO_VUELO=3;
	
	public static final int VUELO_DIRECTO=4;
	public static final int VUELO_RASTREO=4;
	
	public float secuenciaImpacto=0;
	
	private float incrementoAnimacion = 0.03f;
	
	private boolean animando;
	
    /** Creates new Disparo */
    public Disparo(SimpleVector aIni, SimpleVector aDir, ManejadorSonidos aSoundManager, Object3D modeloBala, float avelocidad, int adanio, World mundo, Camera cam) {
        posicion=aIni;
        direccion=aDir;
        
        SoundManager=aSoundManager;
        animando=false;
        
        bala=new Bullet (modeloBala, avelocidad, adanio);
        bala.rotateY((float)(Math.PI)/2);
        bala.rotateMesh();
        bala.setRotationMatrix(cam.getBack().invert3x3());
        mundo.addObject(bala);
        bala.build();
        bala.setAdditionalColor(new Color (80, 80, 200));
        bala.translate(posicion);

    }
    
    public Disparo(SimpleVector aIni, SimpleVector aDir, ManejadorSonidos aSoundManager, Bullet aBala, World mundo, Camera cam) {
        posicion=aIni;
        direccion=aDir;
        
        SoundManager=aSoundManager;
        animando=false;
        
        //bala=new Bullet (aBala);
        bala=aBala;
        bala.rotateY((float)(Math.PI)/2);
        bala.rotateMesh();
        bala.setRotationMatrix(cam.getBack().invert3x3());
        
        mundo.addObject(bala);
        bala.build();
        bala.setAdditionalColor(new Color (80, 80, 200));
        bala.translate(posicion);
    }
    
    public Disparo(Disparo dis, World mundo, Camera cam) {
        posicion=dis.getPosicionInicial();
        direccion=dis.getDireccion();
        
        SoundManager=dis.getSoundManager ();
        animando=false;
        
        bala=new Bullet (dis.getBala());//TODO COMPROBAR SI NO TOCA HACER UN CONSTRUCTOR POR COPIA
        bala.rotateY((float)(Math.PI)/2);
        bala.rotateMesh();
        bala.setRotationMatrix(cam.getBack().invert3x3());
        
        mundo.addObject(bala);
        bala.build();
        bala.setAdditionalColor(new Color (80, 80, 200));
        bala.translate(posicion);
        
    }

    public void play (SimpleVector camara, int tipoSonido) {
    	
    	String pathSonido="";
    	if (tipoSonido==SONIDO_INICIAL) pathSonido=bala.getSonidoInicio();
    	if (tipoSonido==SONIDO_IMPACTO) pathSonido=bala.getSonidoImpacto();
    	if (tipoSonido==SONIDO_VUELO) pathSonido=bala.getSonidoVuelo();
    	SoundManager.play3D(posicion, camara, pathSonido);
    }
    
    public void setDireccion (SimpleVector adireccion) {
    	direccion=adireccion;
    }
    
    public SimpleVector getDireccion () {
    	return direccion;
    }
    
    public void setPosicion (SimpleVector aposicion) {
    	posicion=aposicion;
    }
    
    public SimpleVector getPosicionInicial () {
    	return posicion;
    }
    
    public ManejadorSonidos getSoundManager () {
    	return SoundManager;
    }
    
    public Bullet getBala () {
    	return bala;
    }
    
    public SimpleVector calcularNuevaPosicionDirecta () {
    	
    	SimpleVector destino;
    	float distancia=bala.getVelocidad();
    	destino=new SimpleVector (direccion);
    	destino.scalarMul(distancia);
    	destino.add (posicion);
    	return destino;
    }
    
    /**
     * Mueve la bala en forma directa y reasjusta su posicion
     * 
     * @return true si la bala sigue en movimiento
     * @return false si la bala ha impactado algun objeto
     */
    public boolean avanzarDirecto () {
    	//TODO detectar las colisiones de la bala
    	posicion=calcularNuevaPosicionDirecta();
    	float distancia=bala.getVelocidad();
    	SimpleVector destino=new SimpleVector (direccion);
    	destino.scalarMul(distancia);
    	//System.out.println ("Moviendo la bala hacia: "+destino);
    	bala.translate(destino);
    	
    	
    	if (bala.checkForCollisionEllipsoid (destino, new SimpleVector (10, 10, 10), 5).calcSub(destino).length()>0.3){
    		System.out.println ("\nEL Check for Collision encontro una colision\n");
    		return false;
    	}
    	/*
    	if (bala.checkForCollision(destino, distancia)!=Object3D.NO_OBJECT){
    		System.out.println ("\nEL Check for Collision encontro una colision\n");
    		return false;
    	}*/
    	
    	return true;
    }
    
    /**
     * TODO hacer el metodo para animar la parte de la explosion que le toca en la iteracion
     */
    public boolean AnimarExplosion () {
    	System.out.println ("Animando una explosion de impacto de bala");
    	secuenciaImpacto=secuenciaImpacto+incrementoAnimacion;
    	if (secuenciaImpacto>1) return false;
    	else {
    		bala.animate(secuenciaImpacto);
    		return true;
    	}
    }
    
    public void setBulletListener (BulletListener BL) {
    	bala.addCollisionListener(BL);
    }
    
    public boolean estaAnimando () {
    	return animando;
    }
    
    public void setAnimando (boolean estaAnimando) {
    	animando=estaAnimando;
    }
}
